#ifndef INC_CAVE_H
#define INC_CAVE_H

#include <vector>
#include <functional>
#include <list>
#include <set>
using namespace std;

#include "coord.h"
#include "tile.h"
#include "object.h"

struct Line;

enum CavernType {
    DissolvedCavernType,
    SurfaceCavernType
};
struct CavernData {
    Coord topleft;
    int w;
    int h;
    CavernType type;
    bool generated;
    CavernData(Coord topleft, int w, int h, CavernType type) : topleft(topleft), w(w), h(h), type(type), generated(false) {}
};

class Cavern {
    protected:
	int w;
	int h;
	Array<char> explored;
    public:
	virtual Tile getTile(Coord c) = 0;
	void setExplored(const Coord& c);
	bool isExplored(const Coord& c);
	Cavern(CavernData data);
	virtual ~Cavern() {}
};

class SurfaceCavern : public Cavern {
    public:
	Tile getTile(Coord c);
	SurfaceCavern(CavernData data) : Cavern(data) {}
};

class TestCavern : public Cavern {
    public:
	Tile getTile(Coord c);
	TestCavern(CavernData data) : Cavern(data) {}
};

class Cave {
    static const int NO_CAV = -1;

    int cavs;
    vector<CavernData> mapdata;
    vector<Cavern*> map;
    int cavnumOfCoord(Coord c);

    void generateCavern(int cavnum);
    void populateCavern(int cavnum);

    public:
	ObjectMap objectmap;

	Tile getTile(Coord c);
	TileDef getTileDef(Coord c);
	bool cellPassable(Coord c, bool ignoreObjects=false);
	bool canPass(Coord c, Coord dir, bool ignoreObjects=false, bool flying=false);
	bool contains_if(Coord c, bool (*pred)(const Object&));
	Objects objectsInCell(Coord c);
	void deleteObject(Object* o);
	void setExplored(const Coord& c);
	bool isExplored(const Coord& c);
	Cave();
	~Cave();
};

extern Cave cave;

#endif /* INC_CAVE_H */
