#ifndef INC_COORD_H
#define INC_COORD_H

#include <stdlib.h>
#include <math.h>
#include <vector>
using namespace std;

struct Coord {
    int x;
    int y;

    Coord operator+ (const Coord& other) const {
	return Coord(x + other.x, y + other.y);
    }
    Coord operator- (const Coord& other) const {
	return Coord(x - other.x, y - other.y);
    }
    Coord operator+= (const Coord& other) {
	x += other.x;
	y += other.y;
	return *this;
    }
    Coord operator-= (const Coord& other) {
	x -= other.x;
	y -= other.y;
	return *this;
    }
    Coord operator* (int m) {
	return Coord(m*(x),m*(y));
    }
    Coord operator*= (int m) {
	x *= m;
	y *= m;
	return *this;
    }
    Coord operator/ (float d) {
	return Coord(int(x/d),int(y/d));
    }
    Coord operator/= (float d) {
	x = int(x/d);
	y = int(y/d);
	return *this;
    }
    Coord operator- () const {
	return Coord(0,0) - *this;
    }
    bool operator== (const Coord& other) const {
	return (x == other.x && y == other.y);
    }
    bool operator!= (const Coord& other) const {
	return (!((*this) == other));
    }
    bool operator< (const Coord& other) const {
	return (x < other.x || (x == other.x && y < other.y));
    }

    int lenDiamond() const {
	return x+y;
    }
    int sqLen() const {
	return x*x+y*y;
    }

    float length() const {
	return sqrt(sqLen());
    }

    Coord rotated(int rot) const;

    Coord(int x=0, int y=0) : x(x), y(y) {}
};

extern vector<Coord> orthDirs;
extern vector<Coord> dirs;

int dirIndex(const Coord& dir);
Coord orthDirTowards(const Coord& c);
Coord dirTowards(const Coord& c);

template <class TYPE> class Array {
    private:
	vector< vector< TYPE > > data;
    public:
	TYPE& operator() (const Coord& c) {
	    return data[c.y][c.x];
	}
	const TYPE& operator() (const Coord& c) const {
	    return data[c.y][c.x];
	}
	void init(int w, int h, TYPE init) {
	    data.clear();
	    for( int j=0; j<h; j++ ) {
		vector<TYPE> line;
		for( int i=0; i<w; i++ ) {
		    line.push_back(init);
		}
		data.push_back(line);
	    }
	}
	bool in_bounds(const Coord& c) {
	    return (0 <= c.y && c.y < int(data.size()) && 0 <= c.x && c.x < int(data[0].size()));
	}
};

#endif /* INC_COORD_H */
