#ifndef INC_DISSOLVE_H
#define INC_DISSOLVE_H

#include "cave.h"
#include "coord.h"

typedef vector< vector<Tile> > TileMap;
typedef vector< vector<int> > intMap;

struct drip
{
    static bool is_dead(const drip& drip)
	{ return drip.dead; }

    static const int PRECISION = 10;
    Coord pos;
    Coord vel;
    Coord error;
    bool dead;

    drip( Coord pos, Coord vel=Coord(0,0), Coord error=Coord(0,0) )
	: pos(pos), vel(vel), error(error), dead(false) {}

    void accelerate(int gravity, int max_vel,
	    const Array<int>& height,
	    const Array<int>& numdrips);
};

struct dripper
{
    Coord source;
    Coord initVel;
    unsigned drip_life;
    unsigned max_drips;
    bool dried;

    vector<drip> drips;

    dripper( Coord source, Coord initVel = Coord(0,0) );

    void make_drip();
};

class dissolving_cavern;

class DissolvedCavern : public Cavern {
    Array< Tile > map;

    list<dripper> drippers;
    Coord source;
    Coord sourceDir;
    Coord lavaPos;
    int getMinHeight(Coord c);
    void dissolve();
    void setTile(const dissolving_cavern& cavern, Coord c);
    void traceEdge(const dissolving_cavern& cavern, Coord c);
    void floodLava(const dissolving_cavern& cavern, Coord c);
    bool acceptable();

    int drip_rate;
    unsigned short init_height;
    int init_hardness;
    int init_hardness_variation;
    int dissolve_vert;
    int gravity;
    int hard_areas;
    int encase;

    public:
	Tile getTile(Coord c);
	DissolvedCavern(CavernData data);
};


#endif /* INC_DISSOLVE_H */
