#include <curses.h>
#include <ctime>
#include <signal.h>

#include "anim.h"
#include "coord.h"
#include "cave.h"
#include "tile.h"
#include "object.h"
#include "view.h"
#include "info.h"
#include "event.h"
#include "clock.h"
#include "player.h"
#include "glyph.h"
#include "messages.h"
#include "constants.h"

Cave cave;

EventQueue eventQueue;

Player player = Player(Coord(200,200), Glyph('@', WHITE));

Clock gameClock = { 0 };

GameView gameView(&cave, Coord(0,0), FOV_RAD);

Messages logroll;

Anim anim;

InfoPanel infoPanel;
HelpPanel helpPanel;

Indicators indicators;

void sigint_handler(int signum) {
    if (signum == SIGINT) {
	endwin();
	exit(0);
    }
}

int main(int argc, char** argv) {
    initscr();
    noecho();
    curs_set(0);
    start_color();
    noecho();
    cbreak();
    for( int bg=0; bg<8; bg++ )
	for( int fg=0; fg<8; fg++ )
	    if ( fg>0 || bg>0 )
		init_pair(bg*8 + fg, fg, bg);
    init_pair(63, COLOR_BLACK, COLOR_BLACK);

    signal(SIGINT, sigint_handler);
    setGeometry();

    cave.objectmap.insert(&player);
    gameView.setFocus(player.getPos());
    gameView.drawAll();

    eventQueue.add(new InteractEvent(0, &player));
    eventQueue.add(new LagIncreaseEvent(LAG_INC_TIME, 10));

    bool quit = false;
    while( !quit )
	eventQueue.processNext();
    endwin();
}
