#ifndef INC_OBJECT_H
#define INC_OBJECT_H

#include <vector>
#include <set>
#include <map>
using namespace std;

#include "coord.h"
#include "glyph.h"

enum Collectable {
    NOT_COLLECTABLE = 0,
    COLLECTABLE_FUEL
};

struct ObjectMap;
class Object {
    private:
	// pos and setPos are private, with ObjectMap as a friend, for a
	// reason: if the pos of a mapped object is changed without going via
	// the ObjectMap methods, its ordering might be messed up and bad
	// things would happen!
	Coord pos;
	void setPos(Coord c) { pos=c; }
    protected:
	Glyph glyph;

    public:
	Coord getPos() const { return pos; }

	void move(Coord delta);

	virtual Collectable collectable() const { return NOT_COLLECTABLE; }

	virtual const char* describe() const { return ""; }

	virtual void hit(int damage=1, const char* message="") {}

	virtual bool isThreat() const { return false; }

	virtual bool isStatic() const { return false; }

	virtual Glyph getGlyph() const { return glyph; }

	Object(Coord pos, Glyph glyph) : pos(pos), glyph(glyph) {}

	virtual Object* clone() =0;

	virtual ~Object() {}

    friend struct ObjectMap;
};

class Fuel : public Object {
    public:
	Collectable collectable() const { return COLLECTABLE_FUEL; }
	bool isStatic() const { return true; }
	const char* describe() const { return
	    "Geological good fortune has left a small deposit of usable fuel here."; }
	Fuel(Coord pos) : Object(pos, Glyph('*', GREEN)) {}
	Object* clone() { return new Fuel(*this); }
};

bool isObstruction(const Object& o);
bool isStaticObject(const Object& o);
bool isStaticObstruction(const Object& o);

typedef set<Object*> Objects;

struct ObjectMap : public multimap<Coord,Object*> {
    Objects objectsInCell(Coord c);
    void moveObject(Object* o, Coord to);
    void insert(Object* o) {
	multimap<Coord,Object*>::insert(pair<Coord,Object*>(o->getPos(),o)); }
    void erase(Object* o);
    void erase(Coord c) { multimap<Coord,Object*>::erase(c); }
};

#endif /* INC_OBJECT_H */
