#ifndef INC_SHAPES_H
#define INC_SHAPES_H

#include <iterator>
#include <list>
using namespace std;

#include "coord.h"

class lineIterator;

struct Line {
    // Bresenham line
    Coord s, e;

    int dist(Coord c) const;
    Line normalTo(Coord c) const;
    int len() const;
    bool contains(Coord c) const;
    int intersectsLine(Line other) const;
    lineIterator line() const;

    Coord operator[](int n) const;

    Line(Coord s, Coord e) : s(s), e(e) {}
};

class lineIterator : public std::iterator<bidirectional_iterator_tag, Coord> {
    Coord c;
    int d;
    int error;

    Coord mainDir;
    Coord offDir;
    int slope;
    int dist;

    public:
	operator bool() const;

	lineIterator& operator++();
	lineIterator& operator--();

	lineIterator operator++(int);
	lineIterator operator--(int);

	const Coord& operator*() const;

	lineIterator(Coord s, Coord e);
};

struct rectIterator : public iterator<forward_iterator_tag, Coord> {
    Coord s,e;
    Coord c;
    public:
	rectIterator() {}
	rectIterator(const Coord& s, const Coord& e) : s(s), e(e), c(s) {}
	rectIterator(const Coord& s, int w, int h) : s(s), e(s+Coord(w,h)), c(s) {}
	operator bool() const;
	Coord operator*() const;
	const Coord* operator->() const;
	rectIterator operator++();
	rectIterator operator++(int);
};

struct discIterator : public iterator<forward_iterator_tag, Coord> {
    Coord centre;
    int rsq;
    rectIterator ri;
    public:
	discIterator(const Coord& centre, int r) : centre(centre), rsq(r*r) {
	    ri = rectIterator(centre - Coord(r,r), centre + Coord(r,r));
	    ri++;
	}
	operator bool() const { return ri; }
	Coord operator*() const { return *ri; }
	discIterator operator++();
	discIterator operator++(int);
};

class Pattern {
    vector<Coord> east;
    vector<Coord> northeast;
    
    public:
	vector<Coord> place(Coord origin, Coord dir) const;

	Pattern(const char* epat, const char* nepat);
};
#endif /* INC_SHAPES_H */
